@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -121,8 +121,7 @@ void au_xigen_inc(struct inode *inode)
 	pos = inode->i_ino;
 	pos *= sizeof(igen);
 	igen = inode->i_generation + 1;
-	sz = xino_fwrite(sbinfo->si_xwrite, sbinfo->si_xigen, &igen,
-			 sizeof(igen), &pos);
+	sz = xino_fwrite(sbinfo->si_xigen, &igen, sizeof(igen), &pos);
 	if (sz == sizeof(igen))
 		return; /* success */
 
@@ -164,10 +163,10 @@ int au_xigen_new(struct inode *inode)
 	if (vfsub_f_size_read(file)
 	    < pos + sizeof(inode->i_generation)) {
 		inode->i_generation = atomic_inc_return(&sbinfo->si_xigen_next);
-		sz = xino_fwrite(sbinfo->si_xwrite, file, &inode->i_generation,
+		sz = xino_fwrite(file, &inode->i_generation,
 				 sizeof(inode->i_generation), &pos);
 	} else
-		sz = xino_fread(sbinfo->si_xread, file, &inode->i_generation,
+		sz = xino_fread(file, &inode->i_generation,
 				sizeof(inode->i_generation), &pos);
 	if (sz == sizeof(inode->i_generation))
 		goto out; /* success */
@@ -407,7 +406,7 @@ static struct dentry *au_lkup_by_ino(struct path *path, ino_t ino,
 
 	/* do not call vfsub_lkup_one() */
 	dir = d_inode(parent);
-	dentry = vfsub_lookup_one_len_unlocked(arg.name, parent, arg.namelen);
+	dentry = vfsub_lookup_one_len_unlocked(arg.name, path, arg.namelen);
 	AuTraceErrPtr(dentry);
 	if (IS_ERR(dentry))
 		goto out_name;
@@ -795,14 +794,8 @@ static int aufs_commit_metadata(struct inode *inode)
 	f = h_inode->i_sb->s_export_op->commit_metadata;
 	if (f)
 		err = f(h_inode);
-	else {
-		struct writeback_control wbc = {
-			.sync_mode	= WB_SYNC_ALL,
-			.nr_to_write	= 0 /* metadata only */
-		};
-
-		err = sync_inode(h_inode, &wbc);
-	}
+	else
+		err = sync_inode_metadata(h_inode, /*wait*/1);
 
 	au_cpup_attr_timesizes(inode);
 	ii_write_unlock(inode);
