@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -512,18 +512,20 @@ int au_test_ro(struct super_block *sb, aufs_bindex_t bindex,
 	return err;
 }
 
-int au_test_h_perm(struct inode *h_inode, int mask)
+int au_test_h_perm(struct user_namespace *h_userns, struct inode *h_inode,
+		   int mask)
 {
 	if (uid_eq(current_fsuid(), GLOBAL_ROOT_UID))
 		return 0;
-	return inode_permission(h_inode, mask);
+	return inode_permission(h_userns, h_inode, mask);
 }
 
-int au_test_h_perm_sio(struct inode *h_inode, int mask)
+int au_test_h_perm_sio(struct user_namespace *h_userns, struct inode *h_inode,
+		       int mask)
 {
 	if (au_test_nfs(h_inode->i_sb)
 	    && (mask & MAY_WRITE)
 	    && S_ISDIR(h_inode->i_mode))
 		mask |= MAY_READ; /* force permission check */
-	return au_test_h_perm(h_inode, mask);
+	return au_test_h_perm(h_userns, h_inode, mask);
 }
