@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ static inline void au_kfree_small(const void *p)
 static inline int au_kmidx_sub(size_t sz, size_t new_sz)
 {
 #ifndef CONFIG_SLOB
-	return kmalloc_index(sz) - kmalloc_index(new_sz);
+	return __kmalloc_index(sz, false) - __kmalloc_index(new_sz, false);
 #else
 	return -1; /* SLOB is untested */
 #endif
