@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2014-2020 Junjiro R. Okajima
+ * Copyright (C) 2014-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #include <linux/fs.h>
 #include "aufs.h"
 
-struct posix_acl *aufs_get_acl(struct inode *inode, int type)
+struct posix_acl *aufs_get_acl(struct inode *inode, int type, bool rcu)
 {
 	struct posix_acl *acl;
 	int err;
@@ -31,12 +31,16 @@ struct posix_acl *aufs_get_acl(struct inode *inode, int type)
 	struct inode *h_inode;
 	struct super_block *sb;
 
+	acl = ERR_PTR(-ECHILD);
+	if (rcu)
+		goto out;
+
 	acl = NULL;
 	sb = inode->i_sb;
 	si_read_lock(sb, AuLock_FLUSH);
 	ii_read_lock_child(inode);
 	if (!(sb->s_flags & SB_POSIXACL))
-		goto out;
+		goto unlock;
 
 	bindex = au_ibtop(inode);
 	h_inode = au_h_iptr(inode, bindex);
@@ -45,7 +49,7 @@ struct posix_acl *aufs_get_acl(struct inode *inode, int type)
 			 != (inode->i_mode & S_IFMT)))) {
 		err = au_busy_or_stale();
 		acl = ERR_PTR(err);
-		goto out;
+		goto unlock;
 	}
 
 	/* always topmost only */
@@ -55,15 +59,17 @@ struct posix_acl *aufs_get_acl(struct inode *inode, int type)
 	else
 		set_cached_acl(inode, type, acl);
 
-out:
+unlock:
 	ii_read_unlock(inode);
 	si_read_unlock(sb);
 
+out:
 	AuTraceErrPtr(acl);
 	return acl;
 }
 
-int aufs_set_acl(struct inode *inode, struct posix_acl *acl, int type)
+int aufs_set_acl(struct user_namespace *userns, struct inode *inode,
+		 struct posix_acl *acl, int type)
 {
 	int err;
 	ssize_t ssz;
