@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,19 +32,31 @@
 
 /* to debug easier, do not make them inlined functions */
 #define AuRwMustNoWaiters(rw)	AuDebugOn(rwsem_is_contended(rw))
+
+#ifdef CONFIG_LOCKDEP
 /* rwsem_is_locked() is unusable */
-#define AuRwMustReadLock(rw)	AuDebugOn(!lockdep_recursing(current) \
-					  && debug_locks \
+#define AuRwMustReadLock(rw)	AuDebugOn(IS_ENABLED(CONFIG_LOCKDEP)	\
+					  && !lockdep_recursing(current) \
+					  && debug_locks		\
 					  && !lockdep_is_held_type(rw, 1))
-#define AuRwMustWriteLock(rw)	AuDebugOn(!lockdep_recursing(current) \
-					  && debug_locks \
+#define AuRwMustWriteLock(rw)	AuDebugOn(IS_ENABLED(CONFIG_LOCKDEP)	\
+					  && !lockdep_recursing(current) \
+					  && debug_locks		\
 					  && !lockdep_is_held_type(rw, 0))
-#define AuRwMustAnyLock(rw)	AuDebugOn(!lockdep_recursing(current) \
-					  && debug_locks \
+#define AuRwMustAnyLock(rw)	AuDebugOn(IS_ENABLED(CONFIG_LOCKDEP)	\
+					  && !lockdep_recursing(current) \
+					  && debug_locks		\
 					  && !lockdep_is_held(rw))
-#define AuRwDestroy(rw)		AuDebugOn(!lockdep_recursing(current) \
-					  && debug_locks \
+#define AuRwDestroy(rw)		AuDebugOn(IS_ENABLED(CONFIG_LOCKDEP)	\
+					  && !lockdep_recursing(current) \
+					  && debug_locks		\
 					  && lockdep_is_held(rw))
+#else
+#define AuRwMustReadLock(rw)	do {} while (0)
+#define AuRwMustWriteLock(rw)	do {} while (0)
+#define AuRwMustAnyLock(rw)	do {} while (0)
+#define AuRwDestroy(rw)		do {} while (0)
+#endif
 
 #define au_rw_init(rw)	init_rwsem(rw)
 
