@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,6 +58,11 @@ int vfsub_update_h_iattr(struct path *h_path, int *did)
 	struct kstat st;
 	struct super_block *h_sb;
 
+	/*
+	 * Always needs h_path->mnt for LSM or FUSE branch.
+	 */
+	AuDebugOn(!h_path->mnt);
+
 	/* for remote fs, leave work for its getattr or d_revalidate */
 	/* for bad i_attr fs, handle them in aufs_getattr() */
 	/* still some fs may acquire i_mutex. we need to skip them */
@@ -166,38 +171,38 @@ int vfsub_kern_path(const char *name, unsigned int flags, struct path *path)
 }
 
 struct dentry *vfsub_lookup_one_len_unlocked(const char *name,
-					     struct dentry *parent, int len)
+					     struct path *ppath, int len)
 {
-	struct path path = {
-		.mnt = NULL
-	};
+	struct path path;
 
-	path.dentry = lookup_one_len_unlocked(name, parent, len);
+	path.dentry = lookup_one_len_unlocked(name, ppath->dentry, len);
 	if (IS_ERR(path.dentry))
 		goto out;
-	if (d_is_positive(path.dentry))
+	if (d_is_positive(path.dentry)) {
+		path.mnt = ppath->mnt;
 		vfsub_update_h_iattr(&path, /*did*/NULL); /*ignore*/
+	}
 
 out:
 	AuTraceErrPtr(path.dentry);
 	return path.dentry;
 }
 
-struct dentry *vfsub_lookup_one_len(const char *name, struct dentry *parent,
+struct dentry *vfsub_lookup_one_len(const char *name, struct path *ppath,
 				    int len)
 {
-	struct path path = {
-		.mnt = NULL
-	};
+	struct path path;
 
 	/* VFS checks it too, but by WARN_ON_ONCE() */
-	IMustLock(d_inode(parent));
+	IMustLock(d_inode(ppath->dentry));
 
-	path.dentry = lookup_one_len(name, parent, len);
+	path.dentry = lookup_one_len(name, ppath->dentry, len);
 	if (IS_ERR(path.dentry))
 		goto out;
-	if (d_is_positive(path.dentry))
+	if (d_is_positive(path.dentry)) {
+		path.mnt = ppath->mnt;
 		vfsub_update_h_iattr(&path, /*did*/NULL); /*ignore*/
+	}
 
 out:
 	AuTraceErrPtr(path.dentry);
@@ -207,7 +212,7 @@ out:
 void vfsub_call_lkup_one(void *args)
 {
 	struct vfsub_lkup_one_args *a = args;
-	*a->errp = vfsub_lkup_one(a->name, a->parent);
+	*a->errp = vfsub_lkup_one(a->name, a->ppath);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -244,6 +249,7 @@ int vfsub_create(struct inode *dir, struct path *path, int mode, bool want_excl)
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -253,9 +259,10 @@ int vfsub_create(struct inode *dir, struct path *path, int mode, bool want_excl)
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_create(dir, path->dentry, mode, want_excl);
+	err = vfs_create(userns, dir, path->dentry, mode, want_excl);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = *path;
@@ -277,6 +284,7 @@ int vfsub_symlink(struct inode *dir, struct path *path, const char *symname)
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -286,9 +294,10 @@ int vfsub_symlink(struct inode *dir, struct path *path, const char *symname)
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_symlink(dir, path->dentry, symname);
+	err = vfs_symlink(userns, dir, path->dentry, symname);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = *path;
@@ -310,6 +319,7 @@ int vfsub_mknod(struct inode *dir, struct path *path, int mode, dev_t dev)
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -319,9 +329,10 @@ int vfsub_mknod(struct inode *dir, struct path *path, int mode, dev_t dev)
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_mknod(dir, path->dentry, mode, dev);
+	err = vfs_mknod(userns, dir, path->dentry, mode, dev);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = *path;
@@ -354,6 +365,7 @@ int vfsub_link(struct dentry *src_dentry, struct inode *dir, struct path *path,
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -368,9 +380,10 @@ int vfsub_link(struct dentry *src_dentry, struct inode *dir, struct path *path,
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_link(src_dentry, dir, path->dentry, delegated_inode);
+	err = vfs_link(src_dentry, userns, dir, path->dentry, delegated_inode);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = *path;
@@ -396,6 +409,7 @@ int vfsub_rename(struct inode *src_dir, struct dentry *src_dentry,
 		 struct inode **delegated_inode, unsigned int flags)
 {
 	int err;
+	struct renamedata rd;
 	struct path tmp = {
 		.mnt	= path->mnt
 	};
@@ -412,9 +426,16 @@ int vfsub_rename(struct inode *src_dir, struct dentry *src_dentry,
 	if (unlikely(err))
 		goto out;
 
+	rd.old_mnt_userns = mnt_user_ns(path->mnt);
+	rd.old_dir = src_dir;
+	rd.old_dentry = src_dentry;
+	rd.new_mnt_userns = rd.old_mnt_userns;
+	rd.new_dir = dir;
+	rd.new_dentry = path->dentry;
+	rd.delegated_inode = delegated_inode;
+	rd.flags = flags;
 	lockdep_off();
-	err = vfs_rename(src_dir, src_dentry, dir, path->dentry,
-			 delegated_inode, flags);
+	err = vfs_rename(&rd);
 	lockdep_on();
 	if (!err) {
 		int did;
@@ -438,6 +459,7 @@ int vfsub_mkdir(struct inode *dir, struct path *path, int mode)
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -447,9 +469,10 @@ int vfsub_mkdir(struct inode *dir, struct path *path, int mode)
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_mkdir(dir, path->dentry, mode);
+	err = vfs_mkdir(userns, dir, path->dentry, mode);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = *path;
@@ -471,6 +494,7 @@ int vfsub_rmdir(struct inode *dir, struct path *path)
 {
 	int err;
 	struct dentry *d;
+	struct user_namespace *userns;
 
 	IMustLock(dir);
 
@@ -480,9 +504,10 @@ int vfsub_rmdir(struct inode *dir, struct path *path)
 	path->dentry = d;
 	if (unlikely(err))
 		goto out;
+	userns = mnt_user_ns(path->mnt);
 
 	lockdep_off();
-	err = vfs_rmdir(dir, path->dentry);
+	err = vfs_rmdir(userns, dir, path->dentry);
 	lockdep_on();
 	if (!err) {
 		struct path tmp = {
@@ -513,22 +538,17 @@ ssize_t vfsub_read_u(struct file *file, char __user *ubuf, size_t count,
 	return err;
 }
 
-/* todo: kernel_read()? */
 ssize_t vfsub_read_k(struct file *file, void *kbuf, size_t count,
 		     loff_t *ppos)
 {
 	ssize_t err;
-	mm_segment_t oldfs;
-	union {
-		void *k;
-		char __user *u;
-	} buf;
 
-	buf.k = kbuf;
-	oldfs = get_fs();
-	set_fs(KERNEL_DS);
-	err = vfsub_read_u(file, buf.u, count, ppos);
-	set_fs(oldfs);
+	lockdep_off();
+	err = kernel_read(file, kbuf, count, ppos);
+	lockdep_on();
+	AuTraceErr(err);
+	if (err >= 0)
+		vfsub_update_h_iattr(&file->f_path, /*did*/NULL); /*ignore*/
 	return err;
 }
 
@@ -548,17 +568,12 @@ ssize_t vfsub_write_u(struct file *file, const char __user *ubuf, size_t count,
 ssize_t vfsub_write_k(struct file *file, void *kbuf, size_t count, loff_t *ppos)
 {
 	ssize_t err;
-	mm_segment_t oldfs;
-	union {
-		void *k;
-		const char __user *u;
-	} buf;
 
-	buf.k = kbuf;
-	oldfs = get_fs();
-	set_fs(KERNEL_DS);
-	err = vfsub_write_u(file, buf.u, count, ppos);
-	set_fs(oldfs);
+	lockdep_off();
+	err = kernel_write(file, kbuf, count, ppos);
+	lockdep_on();
+	if (err >= 0)
+		vfsub_update_h_iattr(&file->f_path, /*did*/NULL); /*ignore*/
 	return err;
 }
 
@@ -650,6 +665,7 @@ int vfsub_trunc(struct path *h_path, loff_t length, unsigned int attr,
 	int err;
 	struct inode *h_inode;
 	struct super_block *h_sb;
+	struct user_namespace *h_userns;
 
 	if (!h_file) {
 		err = vfsub_truncate(h_path, length);
@@ -661,12 +677,12 @@ int vfsub_trunc(struct path *h_path, loff_t length, unsigned int attr,
 	lockdep_off();
 	sb_start_write(h_sb);
 	lockdep_on();
-	err = locks_verify_truncate(h_inode, h_file, length);
-	if (!err)
-		err = security_path_truncate(h_path);
+	err = security_path_truncate(h_path);
 	if (!err) {
+		h_userns = mnt_user_ns(h_path->mnt);
 		lockdep_off();
-		err = do_truncate(h_path->dentry, length, attr, h_file);
+		err = do_truncate(h_userns, h_path->dentry, length, attr,
+				  h_file);
 		lockdep_on();
 	}
 	lockdep_off();
@@ -695,8 +711,10 @@ static void au_call_vfsub_mkdir(void *args)
 int vfsub_sio_mkdir(struct inode *dir, struct path *path, int mode)
 {
 	int err, do_sio, wkq_err;
+	struct user_namespace *userns;
 
-	do_sio = au_test_h_perm_sio(dir, MAY_EXEC | MAY_WRITE);
+	userns = mnt_user_ns(path->mnt);
+	do_sio = au_test_h_perm_sio(userns, dir, MAY_EXEC | MAY_WRITE);
 	if (!do_sio) {
 		lockdep_off();
 		err = vfsub_mkdir(dir, path, mode);
@@ -731,8 +749,10 @@ static void au_call_vfsub_rmdir(void *args)
 int vfsub_sio_rmdir(struct inode *dir, struct path *path)
 {
 	int err, do_sio, wkq_err;
+	struct user_namespace *userns;
 
-	do_sio = au_test_h_perm_sio(dir, MAY_EXEC | MAY_WRITE);
+	userns = mnt_user_ns(path->mnt);
+	do_sio = au_test_h_perm_sio(userns, dir, MAY_EXEC | MAY_WRITE);
 	if (!do_sio) {
 		lockdep_off();
 		err = vfsub_rmdir(dir, path);
@@ -764,14 +784,16 @@ static void call_notify_change(void *args)
 {
 	struct notify_change_args *a = args;
 	struct inode *h_inode;
+	struct user_namespace *userns;
 
 	h_inode = d_inode(a->path->dentry);
 	IMustLock(h_inode);
 
 	*a->errp = -EPERM;
 	if (!IS_IMMUTABLE(h_inode) && !IS_APPEND(h_inode)) {
+		userns = mnt_user_ns(a->path->mnt);
 		lockdep_off();
-		*a->errp = notify_change(a->path->dentry, a->ia,
+		*a->errp = notify_change(userns, a->path->dentry, a->ia,
 					 a->delegated_inode);
 		lockdep_on();
 		if (!*a->errp)
@@ -828,6 +850,7 @@ static void call_unlink(void *args)
 	struct unlink_args *a = args;
 	struct dentry *d = a->path->dentry;
 	struct inode *h_inode;
+	struct user_namespace *userns;
 	const int stop_sillyrename = (au_test_nfs(d->d_sb)
 				      && au_dcount(d) == 1);
 
@@ -847,8 +870,9 @@ static void call_unlink(void *args)
 		ihold(h_inode);
 	}
 
+	userns = mnt_user_ns(a->path->mnt);
 	lockdep_off();
-	*a->errp = vfs_unlink(a->dir, d, a->delegated_inode);
+	*a->errp = vfs_unlink(userns, a->dir, d, a->delegated_inode);
 	lockdep_on();
 	if (!*a->errp) {
 		struct path tmp = {
