@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: GPL-2.0 */
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -103,20 +103,20 @@ int vfsub_atomic_open(struct inode *dir, struct dentry *dentry,
 int vfsub_kern_path(const char *name, unsigned int flags, struct path *path);
 
 struct dentry *vfsub_lookup_one_len_unlocked(const char *name,
-					     struct dentry *parent, int len);
-struct dentry *vfsub_lookup_one_len(const char *name, struct dentry *parent,
+					     struct path *ppath, int len);
+struct dentry *vfsub_lookup_one_len(const char *name, struct path *ppath,
 				    int len);
 
 struct vfsub_lkup_one_args {
 	struct dentry **errp;
 	struct qstr *name;
-	struct dentry *parent;
+	struct path *ppath;
 };
 
 static inline struct dentry *vfsub_lkup_one(struct qstr *name,
-					    struct dentry *parent)
+					    struct path *ppath)
 {
-	return vfsub_lookup_one_len(name->name, parent, name->len);
+	return vfsub_lookup_one_len(name->name, ppath, name->len);
 }
 
 void vfsub_call_lkup_one(void *args);
@@ -229,22 +229,24 @@ static inline void vfsub_touch_atime(struct vfsmount *h_mnt,
 static inline int vfsub_update_time(struct inode *h_inode,
 				    struct timespec64 *ts, int flags)
 {
-	return update_time(h_inode, ts, flags);
+	return inode_update_time(h_inode, ts, flags);
 	/* no vfsub_update_h_iattr() since we don't have struct path */
 }
 
 #ifdef CONFIG_FS_POSIX_ACL
-static inline int vfsub_acl_chmod(struct inode *h_inode, umode_t h_mode)
+static inline int vfsub_acl_chmod(struct user_namespace *h_userns,
+				  struct inode *h_inode, umode_t h_mode)
 {
 	int err;
 
-	err = posix_acl_chmod(h_inode, h_mode);
+	err = posix_acl_chmod(h_userns, h_inode, h_mode);
 	if (err == -EOPNOTSUPP)
 		err = 0;
 	return err;
 }
 #else
-AuStubInt0(vfsub_acl_chmod, struct inode *h_inode, umode_t h_mode);
+AuStubInt0(vfsub_acl_chmod, struct user_namespace *h_userns,
+	   struct inode *h_inode, umode_t h_mode);
 #endif
 
 long vfsub_splice_to(struct file *in, loff_t *ppos,
@@ -327,24 +329,26 @@ static inline int vfsub_getattr(const struct path *path, struct kstat *st)
 
 /* ---------------------------------------------------------------------- */
 
-static inline int vfsub_setxattr(struct dentry *dentry, const char *name,
+static inline int vfsub_setxattr(struct user_namespace *userns,
+				 struct dentry *dentry, const char *name,
 				 const void *value, size_t size, int flags)
 {
 	int err;
 
 	lockdep_off();
-	err = vfs_setxattr(dentry, name, value, size, flags);
+	err = vfs_setxattr(userns, dentry, name, value, size, flags);
 	lockdep_on();
 
 	return err;
 }
 
-static inline int vfsub_removexattr(struct dentry *dentry, const char *name)
+static inline int vfsub_removexattr(struct user_namespace *userns,
+				    struct dentry *dentry, const char *name)
 {
 	int err;
 
 	lockdep_off();
-	err = vfs_removexattr(dentry, name);
+	err = vfs_removexattr(userns, dentry, name);
 	lockdep_on();
 
 	return err;
