@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 /*
- * Copyright (C) 2005-2020 Junjiro R. Okajima
+ * Copyright (C) 2005-2021 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,18 +61,19 @@ int au_wh_name_alloc(struct qstr *wh, const struct qstr *name)
 /* ---------------------------------------------------------------------- */
 
 /*
- * test if the @wh_name exists under @h_parent.
+ * test if the @wh_name exists under @h_ppath.
  * @try_sio specifies the necessary of super-io.
  */
-int au_wh_test(struct dentry *h_parent, struct qstr *wh_name, int try_sio)
+int au_wh_test(struct user_namespace *h_userns, struct path *h_ppath,
+	       struct qstr *wh_name, int try_sio)
 {
 	int err;
 	struct dentry *wh_dentry;
 
 	if (!try_sio)
-		wh_dentry = vfsub_lkup_one(wh_name, h_parent);
+		wh_dentry = vfsub_lkup_one(wh_name, h_ppath);
 	else
-		wh_dentry = au_sio_lkup_one(wh_name, h_parent);
+		wh_dentry = au_sio_lkup_one(h_userns, wh_name, h_ppath);
 	err = PTR_ERR(wh_dentry);
 	if (IS_ERR(wh_dentry)) {
 		if (err == -ENAMETOOLONG)
@@ -99,16 +100,16 @@ out:
 }
 
 /*
- * test if the @h_dentry sets opaque or not.
+ * test if the @h_path->dentry sets opaque or not.
  */
-int au_diropq_test(struct dentry *h_dentry)
+int au_diropq_test(struct user_namespace *h_userns, struct path *h_path)
 {
 	int err;
 	struct inode *h_dir;
 
-	h_dir = d_inode(h_dentry);
-	err = au_wh_test(h_dentry, &diropq_name,
-			 au_test_h_perm_sio(h_dir, MAY_EXEC));
+	h_dir = d_inode(h_path->dentry);
+	err = au_wh_test(h_userns, h_path, &diropq_name,
+			 au_test_h_perm_sio(h_userns, h_dir, MAY_EXEC));
 	return err;
 }
 
@@ -125,6 +126,8 @@ struct dentry *au_whtmp_lkup(struct dentry *h_parent, struct au_branch *br,
 	/* strict atomic_t is unnecessary here */
 	static unsigned short cnt;
 	struct qstr qs;
+	struct path h_ppath;
+	struct user_namespace *h_userns;
 
 	BUILD_BUG_ON(sizeof(cnt) * 2 > AUFS_WH_TMP_LEN);
 
@@ -148,10 +151,13 @@ struct dentry *au_whtmp_lkup(struct dentry *h_parent, struct au_branch *br,
 	*p++ = '.';
 	AuDebugOn(name + qs.len + 1 - p <= AUFS_WH_TMP_LEN);
 
+	h_ppath.dentry = h_parent;
+	h_ppath.mnt = au_br_mnt(br);
+	h_userns = au_br_userns(br);
 	qs.name = name;
 	for (i = 0; i < 3; i++) {
 		sprintf(p, "%.*x", AUFS_WH_TMP_LEN, cnt++);
-		dentry = au_sio_lkup_one(&qs, h_parent);
+		dentry = au_sio_lkup_one(h_userns, &qs, &h_ppath);
 		if (IS_ERR(dentry) || d_is_negative(dentry))
 			goto out_name;
 		dput(dentry);
@@ -245,21 +251,20 @@ int au_wh_unlink_dentry(struct inode *h_dir, struct path *h_path,
 	return err;
 }
 
-static int unlink_wh_name(struct dentry *h_parent, struct qstr *wh,
-			  struct au_branch *br)
+static int unlink_wh_name(struct path *h_ppath, struct qstr *wh)
 {
 	int err;
-	struct path h_path = {
-		.mnt = au_br_mnt(br)
-	};
+	struct path h_path;
 
 	err = 0;
-	h_path.dentry = vfsub_lkup_one(wh, h_parent);
+	h_path.dentry = vfsub_lkup_one(wh, h_ppath);
 	if (IS_ERR(h_path.dentry))
 		err = PTR_ERR(h_path.dentry);
 	else {
-		if (d_is_reg(h_path.dentry))
-			err = do_unlink_wh(d_inode(h_parent), &h_path);
+		if (d_is_reg(h_path.dentry)) {
+			h_path.mnt = h_ppath->mnt;
+			err = do_unlink_wh(d_inode(h_ppath->dentry), &h_path);
+		}
 		dput(h_path.dentry);
 	}
 
@@ -699,15 +704,17 @@ out:
 static struct dentry *do_diropq(struct dentry *dentry, aufs_bindex_t bindex,
 				unsigned int flags)
 {
-	struct dentry *opq_dentry, *h_dentry;
+	struct dentry *opq_dentry;
 	struct super_block *sb;
 	struct au_branch *br;
+	struct path h_path;
 	int err;
 
 	sb = dentry->d_sb;
 	br = au_sbr(sb, bindex);
-	h_dentry = au_h_dptr(dentry, bindex);
-	opq_dentry = vfsub_lkup_one(&diropq_name, h_dentry);
+	h_path.dentry = au_h_dptr(dentry, bindex);
+	h_path.mnt = au_br_mnt(br);
+	opq_dentry = vfsub_lkup_one(&diropq_name, &h_path);
 	if (IS_ERR(opq_dentry))
 		goto out;
 
@@ -718,11 +725,8 @@ static struct dentry *do_diropq(struct dentry *dentry, aufs_bindex_t bindex,
 			goto out; /* success */
 		}
 	} else {
-		struct path tmp = {
-			.dentry = opq_dentry,
-			.mnt	= au_br_mnt(br)
-		};
-		err = do_unlink_wh(au_h_iptr(d_inode(dentry), bindex), &tmp);
+		h_path.dentry = opq_dentry;
+		err = do_unlink_wh(au_h_iptr(d_inode(dentry), bindex), &h_path);
 		if (!err)
 			au_set_dbdiropq(dentry, -1);
 	}
@@ -750,9 +754,12 @@ struct dentry *au_diropq_sio(struct dentry *dentry, aufs_bindex_t bindex,
 			     unsigned int flags)
 {
 	struct dentry *diropq, *h_dentry;
+	struct user_namespace *h_userns;
 
+	h_userns = au_sbr_userns(dentry->d_sb, bindex);
 	h_dentry = au_h_dptr(dentry, bindex);
-	if (!au_test_h_perm_sio(d_inode(h_dentry), MAY_EXEC | MAY_WRITE))
+	if (!au_test_h_perm_sio(h_userns, d_inode(h_dentry),
+				MAY_EXEC | MAY_WRITE))
 		diropq = do_diropq(dentry, bindex, flags);
 	else {
 		int wkq_err;
@@ -785,11 +792,14 @@ struct dentry *au_wh_lkup(struct dentry *h_parent, struct qstr *base_name,
 	int err;
 	struct qstr wh_name;
 	struct dentry *wh_dentry;
+	struct path h_path;
 
 	err = au_wh_name_alloc(&wh_name, base_name);
 	wh_dentry = ERR_PTR(err);
 	if (!err) {
-		wh_dentry = vfsub_lkup_one(&wh_name, h_parent);
+		h_path.dentry = h_parent;
+		h_path.mnt = au_br_mnt(br);
+		wh_dentry = vfsub_lkup_one(&wh_name, &h_path);
 		au_kfree_try_rcu(wh_name.name);
 	}
 	return wh_dentry;
@@ -824,8 +834,8 @@ struct dentry *au_wh_create(struct dentry *dentry, aufs_bindex_t bindex,
 /* ---------------------------------------------------------------------- */
 
 /* Delete all whiteouts in this directory on branch bindex. */
-static int del_wh_children(struct dentry *h_dentry, struct au_nhash *whlist,
-			   aufs_bindex_t bindex, struct au_branch *br)
+static int del_wh_children(struct path *h_path, struct au_nhash *whlist,
+			   aufs_bindex_t bindex)
 {
 	int err;
 	unsigned long ul, n;
@@ -855,7 +865,7 @@ static int del_wh_children(struct dentry *h_dentry, struct au_nhash *whlist,
 			if (str->len + AUFS_WH_PFX_LEN <= PATH_MAX) {
 				memcpy(p, str->name, str->len);
 				wh_name.len = AUFS_WH_PFX_LEN + str->len;
-				err = unlink_wh_name(h_dentry, &wh_name, br);
+				err = unlink_wh_name(h_path, &wh_name);
 				if (!err)
 					continue;
 				break;
@@ -874,16 +884,15 @@ out:
 
 struct del_wh_children_args {
 	int *errp;
-	struct dentry *h_dentry;
+	struct path *h_path;
 	struct au_nhash *whlist;
 	aufs_bindex_t bindex;
-	struct au_branch *br;
 };
 
 static void call_del_wh_children(void *args)
 {
 	struct del_wh_children_args *a = args;
-	*a->errp = del_wh_children(a->h_dentry, a->whlist, a->bindex, a->br);
+	*a->errp = del_wh_children(a->h_path, a->whlist, a->bindex);
 }
 
 /* ---------------------------------------------------------------------- */
@@ -935,14 +944,18 @@ int au_whtmp_rmdir(struct inode *dir, aufs_bindex_t bindex,
 {
 	int err;
 	unsigned int h_nlink;
-	struct path h_tmp;
+	struct path wh_path;
 	struct inode *wh_inode, *h_dir;
 	struct au_branch *br;
+	struct user_namespace *h_userns;
 
 	h_dir = d_inode(wh_dentry->d_parent); /* dir inode is locked */
 	IMustLock(h_dir);
 
 	br = au_sbr(dir->i_sb, bindex);
+	wh_path.dentry = wh_dentry;
+	wh_path.mnt = au_br_mnt(br);
+	h_userns = au_br_userns(br);
 	wh_inode = d_inode(wh_dentry);
 	inode_lock_nested(wh_inode, AuLsc_I_CHILD);
 
@@ -950,16 +963,15 @@ int au_whtmp_rmdir(struct inode *dir, aufs_bindex_t bindex,
 	 * someone else might change some whiteouts while we were sleeping.
 	 * it means this whlist may have an obsoleted entry.
 	 */
-	if (!au_test_h_perm_sio(wh_inode, MAY_EXEC | MAY_WRITE))
-		err = del_wh_children(wh_dentry, whlist, bindex, br);
+	if (!au_test_h_perm_sio(h_userns, wh_inode, MAY_EXEC | MAY_WRITE))
+		err = del_wh_children(&wh_path, whlist, bindex);
 	else {
 		int wkq_err;
 		struct del_wh_children_args args = {
 			.errp		= &err,
-			.h_dentry	= wh_dentry,
+			.h_path		= &wh_path,
 			.whlist		= whlist,
-			.bindex		= bindex,
-			.br		= br
+			.bindex		= bindex
 		};
 
 		wkq_err = au_wkq_wait(call_del_wh_children, &args);
@@ -969,10 +981,8 @@ int au_whtmp_rmdir(struct inode *dir, aufs_bindex_t bindex,
 	inode_unlock(wh_inode);
 
 	if (!err) {
-		h_tmp.dentry = wh_dentry;
-		h_tmp.mnt = au_br_mnt(br);
 		h_nlink = h_dir->i_nlink;
-		err = vfsub_rmdir(h_dir, &h_tmp);
+		err = vfsub_rmdir(h_dir, &wh_path);
 		/* some fs doesn't change the parent nlink in some cases */
 		h_nlink -= h_dir->i_nlink;
 	}
